# Global imports
import logging

import requests

from easyeda2kicad import __version__

API_ENDPOINT = "https://easyeda.com/api/products/{lcsc_id}/components?version=6.4.19.5"
ENDPOINT_3D_MODEL = "https://easyeda.com/analyzer/api/3dmodel/{uuid}"
ENDPOINT_3D_MODEL_STEP = "https://modules.easyeda.com/qAxj6KHrDKw4blvCG8QJPs7Y/{uuid}"
# ENDPOINT_3D_MODEL_STEP found in https://modules.lceda.cn/smt-gl-engine/0.8.22.6032922c/smt-gl-engine.js : points to the bucket containing the step files.

# ------------------------------------------------------------


class EasyedaApi:
    def __init__(self) -> None:
        self.headers = {
            "Accept-Encoding": "gzip, deflate",
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "User-Agent": f"easyeda2kicad v{__version__}",
        }

    def get_info_from_easyeda_api(self, lcsc_id: str) -> dict:
        r = requests.get(url=API_ENDPOINT.format(lcsc_id=lcsc_id), headers=self.headers)
        api_response = r.json()

        if not api_response or (
            "code" in api_response and api_response["success"] is False
        ):
            logging.debug(f"{api_response}")
            return {}

        return r.json()

    def get_cad_data_of_component(self, lcsc_id: str) -> dict:
        cp_cad_info = self.get_info_from_easyeda_api(lcsc_id=lcsc_id)
        if cp_cad_info == {}:
            return {}
        return cp_cad_info["result"]

    def get_raw_3d_model_obj(self, uuid: str) -> str:
        r = requests.get(
            url=ENDPOINT_3D_MODEL.format(uuid=uuid),
            headers={"User-Agent": self.headers["User-Agent"]},
        )
        if r.status_code != requests.codes.ok:
            logging.error(f"No raw 3D model data found for uuid:{uuid} on easyeda")
            return None
        return r.content.decode()

    def get_step_3d_model(self, uuid: str) -> bytes:
        r = requests.get(
            url=ENDPOINT_3D_MODEL_STEP.format(uuid=uuid),
            headers={"User-Agent": self.headers["User-Agent"]},
        )
        if r.status_code != requests.codes.ok:
            logging.error(f"No step 3D model data found for uuid:{uuid} on easyeda")
            return None
        return r.content



# Global imports
import itertools
import re
import textwrap
from dataclasses import dataclass, field, fields
from enum import Enum, auto
from typing import List

# ---------------------------- FOOTPRINT PART ----------------------------

KI_MODULE_INFO = "(module {package_lib}:{package_name} (layer F.Cu) (tedit {edit})\n"
KI_DESCRIPTION = (
    '\t(descr "{datasheet_link}, generated with easyeda2kicad.py on {date}")\n'
)
KI_TAGS_INFO = '\t(tags "{tag}")\n'
KI_FP_TYPE = "\t(attr {component_type})\n"
KI_REFERENCE = (
    "\t(fp_text reference REF** (at {pos_x} {pos_y}) (layer F.SilkS)\n\t\t(effects"
    " (font (size 1 1) (thickness 0.15)))\n\t)\n"
)
KI_PACKAGE_VALUE = (
    "\t(fp_text value {package_name} (at {pos_x} {pos_y}) (layer F.Fab)\n\t\t(effects"
    " (font (size 1 1) (thickness 0.15)))\n\t)\n"
)
KI_FAB_REF = (
    "\t(fp_text user %R (at 0 0) (layer F.Fab)\n\t\t(effects (font (size 1 1)"
    " (thickness 0.15)))\n\t)\n"
)
KI_END_FILE = ")"

KI_PAD = (
    "\t(pad {number} {type} {shape} (at {pos_x:.2f} {pos_y:.2f} {orientation:.2f})"
    " (size {width:.2f} {height:.2f}) (layers {layers}){drill}{polygon})\n"
)
KI_LINE = (
    "\t(fp_line (start {start_x:.2f} {start_y:.2f}) (end {end_x:.2f} {end_y:.2f})"
    " (layer {layers}) (width {stroke_width:.2f}))\n"
)
KI_HOLE = (
    '\t(pad "" thru_hole circle (at {pos_x:.2f} {pos_y:.2f}) (size {size:.2f}'
    " {size:.2f}) (drill {size:.2f}) (layers *.Cu *.Mask))\n"
)
KI_VIA = (
    '\t(pad "" thru_hole circle (at {pos_x:.2f} {pos_y:.2f}) (size {diameter:.2f}'
    " {diameter:.2f}) (drill {size:.2f}) (layers *.Cu *.Paste *.Mask))\n"
)
KI_CIRCLE = (
    "\t(fp_circle (center {cx:.2f} {cy:.2f}) (end {end_x:.2f} {end_y:.2f}) (layer"
    " {layers}) (width {stroke_width:.2f}))\n"
)
KI_ARC = (
    "\t(fp_arc (start {start_x:.2f} {start_y:.2f}) (end {end_x:.2f} {end_y:.2f}) (angle"
    " {angle:.2f}) (layer {layers}) (width {stroke_width:.2f}))\n"
)
KI_TEXT = (
    "\t(fp_text user {text} (at {pos_x:.2f} {pos_y:.2f} {orientation:.2f}) (layer"
    " {layers}){display}\n\t\t(effects (font (size {font_size:.2f} {font_size:.2f})"
    " (thickness {thickness:.2f})) (justify left{mirror}))\n\t)\n"
)
KI_MODEL_3D = (
    '\t(model "{file_3d}"\n\t\t(offset (xyz {pos_x:.3f} {pos_y:.3f}'
    " {pos_z:.3f}))\n\t\t(scale (xyz 1 1 1))\n\t\t(rotate (xyz {rot_x:.0f} {rot_y:.0f}"
    " {rot_z:.0f}))\n\t)\n"
)


# ---------------------------------------

KI_PAD_SHAPE = {
    "ELLIPSE": "circle",
    "RECT": "rect",
    "OVAL": "oval",
    "POLYGON": "custom",
}
KI_PAD_LAYER = {
    1: "F.Cu F.Paste F.Mask",
    2: "B.Cu B.Paste B.Mask",
    3: "F.SilkS",
    11: "*.Cu *.Paste *.Mask",
    13: "F.Fab",
    15: "Dwgs.User",
}

KI_PAD_LAYER_THT = {
    1: "F.Cu F.Mask",
    2: "B.Cu B.Mask",
    3: "F.SilkS",
    11: "*.Cu *.Mask",
    13: "F.Fab",
    15: "Dwgs.User",
}

KI_LAYERS = {
    1: "F.Cu",
    2: "B.Cu",
    3: "F.SilkS",
    4: "B.SilkS",
    5: "F.Paste",
    6: "B.Paste",
    7: "F.Mask",
    8: "B.Mask",
    10: "Edge.Cuts",
    11: "Edge.Cuts",
    12: "Cmts.User",
    13: "F.Fab",
    14: "B.Fab",
    15: "Dwgs.User",
    101: "F.Fab",
}


# Round all float values contained in the dataclass
def round_float_values(self) -> None:
    for _field in fields(self):
        current_value = getattr(self, _field.name)
        if isinstance(current_value, float):
            setattr(self, _field.name, round(current_value, 2))


# ---------------- PAD ----------------
@dataclass
class KiFootprintPad:
    type: str
    shape: str
    pos_x: float
    pos_y: float
    width: float
    height: float
    layers: str
    number: str
    drill: float
    orientation: float
    polygon: str

    def __post_init__(self) -> None:
        round_float_values(self)


# ---------------- TRACK ----------------
@dataclass
class KiFootprintTrack:
    points_start_x: List[float] = field(default_factory=list)
    points_start_y: List[float] = field(default_factory=list)
    points_end_x: List[float] = field(default_factory=list)
    points_end_y: List[float] = field(default_factory=list)
    stroke_width: float = 0
    layers: str = ""


# ---------------- HOLE ----------------
@dataclass
class KiFootprintHole:
    pos_x: float
    pos_y: float
    size: float

    def __post_init__(self) -> None:
        round_float_values(self)


# ---------------- CIRCLE ----------------
@dataclass
class KiFootprintCircle:
    cx: float
    cy: float
    end_x: float
    end_y: float
    layers: str
    stroke_width: float

    def __post_init__(self) -> None:
        round_float_values(self)


# ---------------- RECTANGLE ----------------
@dataclass
class KiFootprintRectangle(KiFootprintTrack):
    ...
    # points_start_x:List[float] = field(default_factory=list)
    # points_start_y:List[float] = field(default_factory=list)
    # points_end_x:List[float] = field(default_factory=list)
    # points_end_y:List[float] = field(default_factory=list)
    # stroke_width:float = 0
    # layers:str = ''


# ---------------- ARC ----------------
@dataclass
class KiFootprintArc:
    start_x: float
    start_y: float
    end_x: float
    end_y: float
    angle: float
    layers: str
    stroke_width: float

    def __post_init__(self):
        round_float_values(self)


# ---------------- TEXT ----------------
@dataclass
class KiFootprintText:
    pos_x: float
    pos_y: float
    orientation: float
    text: str
    layers: str
    font_size: float
    thickness: float
    display: str
    mirror: str

    def __post_init__(self):
        round_float_values(self)


# ---------------- VIA ----------------
@dataclass
class KiFootprintVia:
    pos_x: float
    pos_y: float
    size: float
    diameter: float

    def __post_init__(self) -> None:
        round_float_values(self)

    # TODO


# ---------------- SOLID REGION ----------------
@dataclass
class KiFootprintSolidRegion:
    name: str = ""
    # TODO


# ---------------- COPPER AREA ----------------
@dataclass
class KiFootprintCopperArea:
    name: str = ""
    # TODO


# ---------------- FOOTPRINT INFO ----------------
@dataclass
class KiFootprintInfo:
    name: str
    fp_type: str


# ---------------- 3D MODEL ----------------
@dataclass
class Ki3dModelBase:
    x: float = 0.0
    y: float = 0.0
    z: float = 0.0


@dataclass
class Ki3dModel:
    name: str
    translation: Ki3dModelBase
    rotation: Ki3dModelBase
    raw_wrl: str = None


# ---------------- FOOTPRINT  ----------------
@dataclass
class KiFootprint:
    info: KiFootprintInfo
    model_3d: Ki3dModel
    pads: List[KiFootprintPad] = field(default_factory=list)
    tracks: List[KiFootprintTrack] = field(default_factory=list)
    vias: List[KiFootprintVia] = field(default_factory=list)
    holes: List[KiFootprintHole] = field(default_factory=list)
    circles: List[KiFootprintCircle] = field(default_factory=list)
    arcs: List[KiFootprintArc] = field(default_factory=list)
    rectangles: List[KiFootprintRectangle] = field(default_factory=list)
    texts: List[KiFootprintText] = field(default_factory=list)
    solid_regions: List[KiFootprintSolidRegion] = field(default_factory=list)
    copper_areas: List[KiFootprintCopperArea] = field(default_factory=list)



"""
Convert 3D model from .obj format to .wrl with colors
"""

from __future__ import annotations

# Global imports
import re
import textwrap

from easyeda2kicad.easyeda.parameters_easyeda import Ee3dModel
from easyeda2kicad.kicad.parameters_kicad_footprint import Ki3dModel

VRML_HEADER = """#VRML V2.0 utf8
# 3D model generated by easyeda2kicad.py (https://github.com/uPesy/easyeda2kicad.py)
"""


def get_materials(obj_data: str) -> dict:

    material_regex = "newmtl .*?endmtl"
    matchs = re.findall(pattern=material_regex, string=obj_data, flags=re.DOTALL)

    materials = {}
    for match in matchs:
        material = {}
        for value in match.splitlines():
            if value.startswith("newmtl"):
                material_id = value.split(" ")[1]
            elif value.startswith("Ka"):
                material["ambient_color"] = value.split(" ")[1:]
            elif value.startswith("Kd"):
                material["diffuse_color"] = value.split(" ")[1:]
            elif value.startswith("Ks"):
                material["specular_color"] = value.split(" ")[1:]
            elif value.startswith("d"):
                # This isn't exactly the same as transparency, is dissolve
                # I.e. part C115366 (SW-TH_SPEF110100) has d=1, and isn't transparent
                material["transparency"] = value.split(" ")[1]

        materials[material_id] = material
    return materials


def get_vertices(obj_data: str) -> list:
    vertices_regex = "v (.*?)\n"
    matchs = re.findall(pattern=vertices_regex, string=obj_data, flags=re.DOTALL)

    return [
        " ".join([str(round(float(coord) / 2.54, 4)) for coord in vertice.split(" ")])
        for vertice in matchs
    ]


def generate_wrl_model(model_3d: Ee3dModel) -> Ki3dModel:
    materials = get_materials(obj_data=model_3d.raw_obj)
    vertices = get_vertices(obj_data=model_3d.raw_obj)

    raw_wrl = VRML_HEADER
    shapes = model_3d.raw_obj.split("usemtl")[1:]
    for shape in shapes:
        lines = shape.splitlines()
        material = materials[lines[0].replace(" ", "")]
        index_counter = 0
        link_dict = {}
        coord_index = []
        points = []
        for line in lines[1:]:
            if len(line) > 0:
                face = [int(index) for index in line.replace("//", "").split(" ")[1:]]
                face_index = []
                for index in face:
                    if index not in link_dict:
                        link_dict[index] = index_counter
                        face_index.append(str(index_counter))
                        points.append(vertices[index - 1])
                        index_counter += 1
                    else:
                        face_index.append(str(link_dict[index]))
                face_index.append("-1")
                coord_index.append(",".join(face_index) + ",")
        points.insert(-1, points[-1])

        shape_str = textwrap.dedent(
            f"""
            Shape{{
                appearance Appearance {{
                    material  Material 	{{
                        diffuseColor {' '.join(material['diffuse_color'])}
                        specularColor {' '.join(material['specular_color'])}
                        ambientIntensity 0.2
                        transparency 0
                        shininess 0.5
                    }}
                }}
                geometry IndexedFaceSet {{
                    ccw TRUE
                    solid FALSE
                    coord DEF co Coordinate {{
                        point [
                            {(", ").join(points)}
                        ]
                    }}
                    coordIndex [
                        {"".join(coord_index)}
                    ]
                }}
            }}"""
        )

        raw_wrl += shape_str

    return Ki3dModel(
        translation=None, rotation=None, name=model_3d.name, raw_wrl=raw_wrl
    )


class Exporter3dModelKicad:
    def __init__(self, model_3d: Ee3dModel):
        self.input = model_3d
        self.output = (
            generate_wrl_model(model_3d=model_3d)
            if model_3d and model_3d.raw_obj
            else None
        )
        self.output_step = model_3d.step

    def export(self, lib_path: str) -> None:
        if self.output:
            with open(
                file=f"{lib_path}.3dshapes/{self.output.name}.wrl",
                mode="w",
                encoding="utf-8",
            ) as my_lib:
                my_lib.write(self.output.raw_wrl)
        if self.output_step:
            with open(
                file=f"{lib_path}.3dshapes/{self.output.name}.step",
                mode="wb",
            ) as my_lib:
                my_lib.write(self.output_step)



# Global imports
import json
import logging

from easyeda2kicad.easyeda.easyeda_api import EasyedaApi
from easyeda2kicad.easyeda.parameters_easyeda import *


def add_easyeda_pin(pin_data: str, ee_symbol: EeSymbol):
    segments = pin_data.split("^^")
    ee_segments = [seg.split("~") for seg in segments]

    pin_settings = EeSymbolPinSettings(
        **dict(zip(EeSymbolPinSettings.__fields__, ee_segments[0][1:]))
    )
    pin_dot = EeSymbolPinDot(
        dot_x=float(ee_segments[1][0]), dot_y=float(ee_segments[1][1])
    )
    pin_path = EeSymbolPinPath(path=ee_segments[2][0], color=ee_segments[2][1])
    pin_name = EeSymbolPinName(
        **dict(zip(EeSymbolPinName.__fields__, ee_segments[3][:]))
    )

    pin_dot_bis = EeSymbolPinDotBis(
        is_displayed=ee_segments[5][0],
        circle_x=float(ee_segments[5][1]),
        circle_y=float(ee_segments[5][2]),
    )
    pin_clock = EeSymbolPinClock(is_displayed=ee_segments[6][0], path=ee_segments[6][1])

    ee_symbol.pins.append(
        EeSymbolPin(
            settings=pin_settings,
            pin_dot=pin_dot,
            pin_path=pin_path,
            name=pin_name,
            dot=pin_dot_bis,
            clock=pin_clock,
        )
    )


def add_easyeda_rectangle(rectangle_data: str, ee_symbol: EeSymbol):
    ee_symbol.rectangles.append(
        EeSymbolRectangle(
            **dict(zip(EeSymbolRectangle.__fields__, rectangle_data.split("~")[1:]))
        )
    )


def add_easyeda_polyline(polyline_data: str, ee_symbol: EeSymbol):
    ee_symbol.polylines.append(
        EeSymbolPolyline(
            **dict(zip(EeSymbolPolyline.__fields__, polyline_data.split("~")[1:]))
        )
    )


def add_easyeda_polygon(polygon_data: str, ee_symbol: EeSymbol):
    ee_symbol.polygons.append(
        EeSymbolPolygon(
            **dict(zip(EeSymbolPolygon.__fields__, polygon_data.split("~")[1:]))
        )
    )


def add_easyeda_path(path_data: str, ee_symbol: EeSymbol):
    ee_symbol.paths.append(
        EeSymbolPath(**dict(zip(EeSymbolPath.__fields__, path_data.split("~")[1:])))
    )


def add_easyeda_circle(circle_data: str, ee_symbol: EeSymbol):
    ee_symbol.circles.append(
        EeSymbolCircle(
            **dict(zip(EeSymbolCircle.__fields__, circle_data.split("~")[1:]))
        )
    )


def add_easyeda_ellipse(ellipse_data: str, ee_symbol: EeSymbol):
    ee_symbol.ellipses.append(
        EeSymbolEllipse(
            **dict(zip(EeSymbolEllipse.__fields__, ellipse_data.split("~")[1:]))
        )
    )


def add_easyeda_arc(arc_data: str, ee_symbol: EeSymbol):
    ee_symbol.arcs.append(
        EeSymbolArc(**dict(zip(EeSymbolArc.__fields__, arc_data.split("~")[1:])))
    )


easyeda_handlers = {
    "P": add_easyeda_pin,
    "R": add_easyeda_rectangle,
    "E": add_easyeda_ellipse,
    "C": add_easyeda_circle,
    "A": add_easyeda_arc,
    "PL": add_easyeda_polyline,
    "PG": add_easyeda_polygon,
    "PT": add_easyeda_path,
    # "PI" : Pie, Elliptical arc seems to be not supported in Kicad
}


class EasyedaSymbolImporter:
    def __init__(self, easyeda_cp_cad_data: dict):
        self.input = easyeda_cp_cad_data
        self.output: EeSymbol = self.extract_easyeda_data(
            ee_data=easyeda_cp_cad_data,
            ee_data_info=easyeda_cp_cad_data["dataStr"]["head"]["c_para"],
        )

    def get_symbol(self) -> EeSymbol:
        return self.output

    def extract_easyeda_data(self, ee_data: dict, ee_data_info: dict) -> EeSymbol:
        new_ee_symbol = EeSymbol(
            info=EeSymbolInfo(
                name=ee_data_info["name"],
                prefix=ee_data_info["pre"],
                package=ee_data_info.get("package", None),
                manufacturer=ee_data_info.get("BOM_Manufacturer", None),
                datasheet=ee_data["lcsc"].get("url", None),
                lcsc_id=ee_data["lcsc"].get("number", None),
                jlc_id=ee_data_info.get("BOM_JLCPCB Part Class", None),
            ),
            bbox=EeSymbolBbox(
                x=float(ee_data["dataStr"]["head"]["x"]),
                y=float(ee_data["dataStr"]["head"]["y"]),
            ),
        )

        for line in ee_data["dataStr"]["shape"]:
            designator = line.split("~")[0]
            if designator in easyeda_handlers:
                easyeda_handlers[designator](line, new_ee_symbol)
            else:
                logging.warning(f"Unknow symbol designator : {designator}")

        return new_ee_symbol


class EasyedaFootprintImporter:
    def __init__(self, easyeda_cp_cad_data: dict):
        self.input = easyeda_cp_cad_data
        self.output = self.extract_easyeda_data(
            ee_data_str=self.input["packageDetail"]["dataStr"],
            ee_data_info=self.input["packageDetail"]["dataStr"]["head"]["c_para"],
            is_smd=self.input.get("SMT")
            and "-TH_" not in self.input["packageDetail"]["title"],
        )

    def get_footprint(self):
        return self.output

    def extract_easyeda_data(
        self, ee_data_str: dict, ee_data_info: dict, is_smd: bool
    ) -> ee_footprint:
        new_ee_footprint = ee_footprint(
            info=EeFootprintInfo(
                name=ee_data_info["package"],
                fp_type="smd" if is_smd else "tht",
                model_3d_name=ee_data_info.get("3DModel"),
            ),
            bbox=EeFootprintBbox(
                x=float(ee_data_str["head"]["x"]),
                y=float(ee_data_str["head"]["y"]),
            ),
            model_3d=None,
        )

        for line in ee_data_str["shape"]:
            ee_designator = line.split("~")[0]
            ee_fields = line.split("~")[1:]

            if ee_designator == "PAD":
                ee_pad = EeFootprintPad(
                    **dict(zip(EeFootprintPad.__fields__, ee_fields[:18]))
                )
                new_ee_footprint.pads.append(ee_pad)
            elif ee_designator == "TRACK":
                ee_track = EeFootprintTrack(
                    **dict(zip(EeFootprintTrack.__fields__, ee_fields))
                )
                new_ee_footprint.tracks.append(ee_track)
            elif ee_designator == "HOLE":
                ee_hole = EeFootprintHole(
                    **dict(zip(EeFootprintHole.__fields__, ee_fields))
                )
                new_ee_footprint.holes.append(ee_hole)
            elif ee_designator == "VIA":
                ee_via = EeFootprintVia(
                    **dict(zip(EeFootprintVia.__fields__, ee_fields))
                )
                new_ee_footprint.vias.append(ee_via)
            elif ee_designator == "CIRCLE":
                ee_circle = EeFootprintCircle(
                    **dict(zip(EeFootprintCircle.__fields__, ee_fields))
                )
                new_ee_footprint.circles.append(ee_circle)
            elif ee_designator == "ARC":
                ee_arc = EeFootprintArc(
                    **dict(zip(EeFootprintArc.__fields__, ee_fields))
                )
                new_ee_footprint.arcs.append(ee_arc)
            elif ee_designator == "RECT":
                ee_rectangle = EeFootprintRectangle(
                    **dict(zip(EeFootprintRectangle.__fields__, ee_fields))
                )
                new_ee_footprint.rectangles.append(ee_rectangle)
            elif ee_designator == "TEXT":
                ee_text = EeFootprintText(
                    **dict(zip(EeFootprintText.__fields__, ee_fields))
                )
                new_ee_footprint.texts.append(ee_text)
            elif ee_designator == "SVGNODE":
                new_ee_footprint.model_3d = Easyeda3dModelImporter(
                    easyeda_cp_cad_data=[line], download_raw_3d_model=False
                ).output

            elif ee_designator == "SOLIDREGION":
                ...
            else:
                logging.warning(f"Unknow footprint designator : {ee_designator}")

        return new_ee_footprint


# ------------------------------------------------------------------------------


class Easyeda3dModelImporter:
    def __init__(self, easyeda_cp_cad_data, download_raw_3d_model: bool):
        self.input = easyeda_cp_cad_data
        self.download_raw_3d_model = download_raw_3d_model
        self.output = self.create_3d_model()

    def create_3d_model(self) -> Union[Ee3dModel, None]:
        ee_data = (
            self.input["packageDetail"]["dataStr"]["shape"]
            if isinstance(self.input, dict)
            else self.input
        )

        if model_3d_info := self.get_3d_model_info(ee_data=ee_data):
            model_3d: Ee3dModel = self.parse_3d_model_info(info=model_3d_info)
            if self.download_raw_3d_model:
                model_3d.raw_obj = EasyedaApi().get_raw_3d_model_obj(uuid=model_3d.uuid)
                model_3d.step = EasyedaApi().get_step_3d_model(uuid=model_3d.uuid)
            return model_3d

        logging.warning("No 3D model available for this component")
        return None

    def get_3d_model_info(self, ee_data: str) -> dict:
        for line in ee_data:
            ee_designator = line.split("~")[0]
            if ee_designator == "SVGNODE":
                raw_json = line.split("~")[1:][0]
                return json.loads(raw_json)["attrs"]
        return {}

    def parse_3d_model_info(self, info: dict) -> Ee3dModel:
        return Ee3dModel(
            name=info["title"],
            uuid=info["uuid"],
            translation=Ee3dModelBase(
                x=info["c_origin"].split(",")[0],
                y=info["c_origin"].split(",")[1],
                z=info["z"],
            ),
            rotation=Ee3dModelBase(
                **dict(zip(Ee3dModelBase.__fields__, info["c_rotation"].split(",")))
            ),
        )



# Global imports
from dataclasses import dataclass, field
from enum import Enum
from typing import List, Union

from pydantic import BaseModel, field_validator

from easyeda2kicad.easyeda.svg_path_parser import parse_svg_path


class EasyedaPinType(Enum):
    unspecified = 0
    _input = 1
    output = 2
    bidirectional = 3
    power = 4


# ------------------------- Symbol -------------------------
class EeSymbolBbox(BaseModel):
    x: float
    y: float


# ---------------- PIN ----------------
class EeSymbolPinSettings(BaseModel):
    is_displayed: bool
    type: EasyedaPinType
    spice_pin_number: str
    pos_x: float
    pos_y: float
    rotation: int
    id: str
    is_locked: bool

    @field_validator("is_displayed", mode="before")
    @classmethod
    def parse_display_field(cls, field: str) -> bool:
        return True if field == "show" else field

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, is_locked: str) -> str:
        return is_locked or False

    @field_validator("rotation", mode="before")
    @classmethod
    def empty_str_rotation(cls, rotation: str) -> str:
        return rotation or 0.0

    @field_validator("type", mode="before")
    @classmethod
    def convert_pin_type(cls, field: str) -> str:
        return (
            EasyedaPinType(int(field or 0))
            if int(field or 0) in EasyedaPinType._value2member_map_
            else EasyedaPinType._unspecified
        )


class EeSymbolPinDot(BaseModel):
    dot_x: float
    dot_y: float


class EeSymbolPinPath(BaseModel):
    path: str
    color: str

    @field_validator("path", mode="before")
    @classmethod
    def tune_path(cls, field: str) -> str:
        return field.replace("v", "h")


class EeSymbolPinName(BaseModel):
    is_displayed: bool
    pos_x: float
    pos_y: float
    rotation: int
    text: str
    text_anchor: str
    font: str
    font_size: float

    @field_validator("font_size", mode="before")
    @classmethod
    def empty_str_font(cls, font_size: str) -> float:
        if isinstance(font_size, str) and "pt" in font_size:
            return float(font_size.replace("pt", ""))
        return font_size or 7.0

    @field_validator("is_displayed", mode="before")
    @classmethod
    def parse_display_field(cls, field: str) -> str:
        return True if field == "show" else field

    @field_validator("rotation", mode="before")
    @classmethod
    def empty_str_rotation(cls, rotation: str) -> str:
        return rotation or 0.0


class EeSymbolPinDotBis(BaseModel):
    is_displayed: bool
    circle_x: float
    circle_y: float

    @field_validator("is_displayed", mode="before")
    @classmethod
    def parse_display_field(cls, field: str) -> str:
        return True if field == "show" else field


class EeSymbolPinClock(BaseModel):
    is_displayed: bool
    path: str

    @field_validator("is_displayed", mode="before")
    @classmethod
    def parse_display_field(cls, field: str) -> str:
        return True if field == "show" else field


@dataclass
class EeSymbolPin:
    settings: EeSymbolPinSettings
    pin_dot: EeSymbolPinDot
    pin_path: EeSymbolPinPath
    name: EeSymbolPinName
    dot: EeSymbolPinDotBis
    clock: EeSymbolPinClock


# ---------------- RECTANGLE ----------------
class EeSymbolRectangle(BaseModel):
    pos_x: float
    pos_y: float
    rx: Union[float, None] = None
    ry: Union[float, None] = None
    width: float
    height: float
    stroke_color: str
    stroke_width: str
    stroke_style: str
    fill_color: str
    id: str
    is_locked: bool

    @field_validator("*", mode="before")
    @classmethod
    def empty_str_to_none(cls, field: str) -> str:
        return field or None


# ---------------- CIRCLE ----------------
class EeSymbolCircle(BaseModel):
    center_x: float
    center_y: float
    radius: float
    stroke_color: str
    stroke_width: str
    stroke_style: str
    fill_color: bool
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    @field_validator("fill_color", mode="before")
    @classmethod
    def parse_background_filling(cls, fill_color: str) -> str:
        return bool(fill_color and fill_color.lower() != "none")


# ---------------- ARC ----------------
class EeSymbolArc(BaseModel):
    path: list
    helper_dots: str
    stroke_color: str
    stroke_width: str
    stroke_style: str
    fill_color: bool
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    @field_validator("fill_color", mode="before")
    @classmethod
    def parse_background_filling(cls, fill_color: str) -> str:
        return bool(fill_color and fill_color.lower() != "none")

    @field_validator("path", mode="before")
    @classmethod
    def convert_svg_path(cls, path: str) -> list:
        return parse_svg_path(svg_path=path)


class EeSymbolEllipse(BaseModel):
    center_x: float
    center_y: float
    radius_x: float
    radius_y: float
    stroke_color: str
    stroke_width: str
    stroke_style: str
    fill_color: bool
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    @field_validator("fill_color", mode="before")
    @classmethod
    def parse_background_filling(cls, fill_color: str) -> str:
        return bool(fill_color and fill_color.lower() != "none")


# ---------------- POLYLINE ----------------
class EeSymbolPolyline(BaseModel):
    points: str
    stroke_color: str
    stroke_width: str
    stroke_style: str
    fill_color: bool
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    @field_validator("fill_color", mode="before")
    @classmethod
    def parse_background_filling(cls, fill_color: str) -> str:
        return bool(fill_color and fill_color.lower() != "none")


# ---------------- POLYGON ----------------
class EeSymbolPolygon(EeSymbolPolyline):
    ...


# ---------------- PATH ----------------
# TODO : EeSymbolPath.paths should be a SVG PATH https://www.w3.org/TR/SVG11/paths.html#PathElement
# TODO : small svg parser and then convert to kicad
# TODO: support bezier curve, currently paths are seen as polygone
class EeSymbolPath(BaseModel):
    paths: str
    stroke_color: str
    stroke_width: str
    stroke_style: str
    fill_color: bool
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    @field_validator("fill_color", mode="before")
    @classmethod
    def parse_background_filling(cls, fill_color: str) -> str:
        return bool(fill_color and fill_color.lower() != "none")

    # @validator("paths", pre=True)
    # def clean_svg_path(cls, paths:str):
    #     return paths.replace("M", "").replace("C","")


# ---------------- SYMBOL ----------------
@dataclass
class EeSymbolInfo:
    name: str = ""
    prefix: str = ""
    package: str = ""
    manufacturer: str = ""
    datasheet: str = ""
    lcsc_id: str = ""
    jlc_id: str = ""


@dataclass
class EeSymbol:
    info: EeSymbolInfo
    bbox: EeSymbolBbox
    pins: List[EeSymbolPin] = field(default_factory=list)
    rectangles: List[EeSymbolRectangle] = field(default_factory=list)
    circles: List[EeSymbolCircle] = field(default_factory=list)
    arcs: List[EeSymbolArc] = field(default_factory=list)
    ellipses: List[EeSymbolEllipse] = field(default_factory=list)
    polylines: List[EeSymbolPolyline] = field(default_factory=list)
    polygons: List[EeSymbolPolygon] = field(default_factory=list)
    paths: List[EeSymbolPath] = field(default_factory=list)


# ------------------------- Footprint -------------------------


def convert_to_mm(dim: float) -> float:
    return float(dim) * 10 * 0.0254


@dataclass
class EeFootprintBbox:

    x: float
    y: float

    def convert_to_mm(self) -> None:
        self.x = convert_to_mm(self.x)
        self.y = convert_to_mm(self.y)


class EeFootprintPad(BaseModel):
    shape: str
    center_x: float
    center_y: float
    width: float
    height: float
    layer_id: int
    net: str
    number: str
    hole_radius: float
    points: str
    rotation: float
    id: str
    hole_length: float
    hole_point: str
    is_plated: bool
    is_locked: bool

    def convert_to_mm(self) -> None:
        self.center_x = convert_to_mm(self.center_x)
        self.center_y = convert_to_mm(self.center_y)
        self.width = convert_to_mm(self.width)
        self.height = convert_to_mm(self.height)
        self.hole_radius = convert_to_mm(self.hole_radius)
        self.hole_length = convert_to_mm(self.hole_length)

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    @field_validator("rotation", mode="before")
    @classmethod
    def empty_str_rotation(cls, field: str) -> str:
        return field or 0.0


class EeFootprintTrack(BaseModel):
    stroke_width: float
    layer_id: int
    net: str
    points: str
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    def convert_to_mm(self) -> None:
        self.stroke_width = convert_to_mm(self.stroke_width)


class EeFootprintHole(BaseModel):
    center_x: float
    center_y: float
    radius: float
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    def convert_to_mm(self) -> None:
        self.center_x = convert_to_mm(self.center_x)
        self.center_y = convert_to_mm(self.center_y)
        self.radius = convert_to_mm(self.radius)


class EeFootprintVia(BaseModel):
    center_x: float
    center_y: float
    diameter: float
    net: str
    radius: float
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    def convert_to_mm(self) -> None:
        self.center_x = convert_to_mm(self.center_x)
        self.center_y = convert_to_mm(self.center_y)
        self.radius = convert_to_mm(self.radius)
        self.diameter = convert_to_mm(self.diameter)


class EeFootprintCircle(BaseModel):
    cx: float
    cy: float
    radius: float
    stroke_width: float
    layer_id: int
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field: str) -> str:
        return field or False

    def convert_to_mm(self) -> None:
        self.cx = convert_to_mm(self.cx)
        self.cy = convert_to_mm(self.cy)
        self.radius = convert_to_mm(self.radius)
        self.stroke_width = convert_to_mm(self.stroke_width)


class EeFootprintRectangle(BaseModel):
    x: float
    y: float
    width: float
    height: float
    stroke_width: float
    id: str
    layer_id: int
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field):
        return False if field == "" else bool(float(field))

    def convert_to_mm(self):
        self.x = convert_to_mm(self.x)
        self.y = convert_to_mm(self.y)
        self.width = convert_to_mm(self.width)
        self.height = convert_to_mm(self.height)


class EeFootprintArc(BaseModel):
    stroke_width: float
    layer_id: int
    net: str
    path: str
    helper_dots: str
    id: str
    is_locked: bool

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field):
        return False if field == "" else field


class EeFootprintText(BaseModel):
    type: str
    center_x: float
    center_y: float
    stroke_width: float
    rotation: int
    miror: str
    layer_id: int
    net: str
    font_size: float
    text: str
    text_path: str
    is_displayed: bool
    id: str
    is_locked: bool

    @field_validator("is_displayed", mode="before")
    @classmethod
    def empty_str_display(cls, field):
        return True if field == "" else field

    @field_validator("is_locked", mode="before")
    @classmethod
    def empty_str_lock(cls, field):
        return False if field == "" else field

    @field_validator("rotation", mode="before")
    @classmethod
    def empty_str_rotation(cls, field):
        return 0.0 if field == "" else field

    def convert_to_mm(self):

        self.center_x = convert_to_mm(self.center_x)
        self.center_y = convert_to_mm(self.center_y)
        self.stroke_width = convert_to_mm(self.stroke_width)
        self.font_size = convert_to_mm(self.font_size)


# ---------------- FOOTPRINT ----------------


@dataclass
class EeFootprintInfo:
    name: str
    fp_type: str
    model_3d_name: str


# ------------------------- 3D MODEL -------------------------
class Ee3dModelBase(BaseModel):
    x: float = 0.0
    y: float = 0.0
    z: float = 0.0

    def convert_to_mm(self) -> None:
        self.x = convert_to_mm(self.x)
        self.y = convert_to_mm(self.y)
        self.z = convert_to_mm(self.z)


@dataclass
class Ee3dModel:
    name: str
    uuid: str
    translation: Ee3dModelBase
    rotation: Ee3dModelBase
    raw_obj: str = None
    step: bytes = None

    def convert_to_mm(self) -> None:
        self.translation.convert_to_mm()
        # self.translation.z = self.translation.z


@dataclass
class ee_footprint:
    info: EeFootprintInfo
    bbox: EeFootprintBbox
    model_3d: Ee3dModel
    pads: List[EeFootprintPad] = field(default_factory=list)
    tracks: List[EeFootprintTrack] = field(default_factory=list)
    holes: List[EeFootprintHole] = field(default_factory=list)
    vias: List[EeFootprintVia] = field(default_factory=list)
    circles: List[EeFootprintCircle] = field(default_factory=list)
    arcs: List[EeFootprintArc] = field(default_factory=list)
    rectangles: List[EeFootprintRectangle] = field(default_factory=list)
    texts: List[EeFootprintText] = field(default_factory=list)



# Global imports
import logging
from pathlib import Path
import re
from textwrap import dedent

log = logging.getLogger(__name__)

from easyeda2kicad.easyeda.parameters_easyeda import (
    EasyedaPinType,
    EeSymbol,
)

ee_pin_type_to_ato_pin_type = {
    EasyedaPinType.unspecified: None,
    EasyedaPinType._input: "input",
    EasyedaPinType.output: "output",
    EasyedaPinType.bidirectional: "bidirectional",
    EasyedaPinType.power: "power",
}
ee_pin_rotation_to_vis_side = {0: "right", 90: "bottom", 180: "left", 270: "top"}


def add_pin_vis(name, pos):
    return f"""
  - name: {name}
    index: 0
    private: false
    port: {pos}"""


def sanitize_name(name: str) -> str:
    # Replace characters based on the replacement_dict
    for symbol, replacement in replacement_dict.items():
        name = name.replace(symbol, f"_{replacement}_")

    # Replace all non-alphanumeric characters (except underscores) with a single underscore
    sanitized = re.sub(r'[^a-zA-Z0-9_]+', '_', name)

    # Remove any leading, trailing, or multiple consecutive underscores
    sanitized = re.sub(r'^_+|_+$|(?<=_)_+', '', sanitized)

    # Check if the first character is a digit, and if so, prepend an underscore
    if sanitized and sanitized[0].isdigit():
        sanitized = f"_{sanitized}"

    return sanitized

replacement_dict = {
    "+": "plus",
    "-": "minus",
    "/": "slash",
    "*": "star",
    "(": "lparen",
    ")": "rparen",
    "[": "lbracket",
    "]": "rbracket",
    "{": "lbrace",
    "}": "rbrace",
    "<": "less",
    ">": "greater",
    "=": "equal",
    "~": "tilde",
    "!": "exclamation",
    "@": "at",
    "#": "hash",
    "$": "dollar",
    "%": "percent",
    "^": "caret",
    "&": "ampersand",
    "|": "pipe",
    "\\": "backslash",
    ":": "colon",
    ";": "semicolon",
    "'": "apostrophe",
    '"': "quote",
    "?": "question",
    ",": "comma",
    ".": "period",
    " ": "space",
    "\t": "tab",
}

def convert_to_ato(
    ee_symbol: EeSymbol, component_id: str, component_name: str, footprint: str
) -> str:
    ato_str = f"component {sanitize_name(component_name)}:\n"
    ato_str += f"    # component {component_name}\n"
    ato_str += f'    footprint = "{footprint}"\n'
    ato_str += f'    lcsc_id = "{component_id}"\n'
    ato_str += "    # pins\n"

    defined_signals = set()
    for ee_pin in ee_symbol.pins:
        # clean the signal name from any non-alphanumeric symbols or leading digits
        signal = sanitize_name(ee_pin.name.text)

        pin = ee_pin.settings.spice_pin_number.replace(" ", "")
        #check if the signal name has already been defined

        if signal in defined_signals:
            #if it has, append the pin number to the signal name
            ato_str += f"    {signal} ~ pin {pin}\n"
        else:
            defined_signals.add(signal)
            ato_str += f"    signal {signal} ~ pin {pin}\n"
        ato_pin_type = ee_pin_type_to_ato_pin_type[ee_pin.settings.type]

    return ato_str + "\n"


class ExporterAto:
    def __init__(self, symbol, component_id: str, component_name: str, footprint: str):
        self.input: EeSymbol = symbol
        self.output = (
            convert_to_ato(
                ee_symbol=self.input,
                component_id=component_id,
                component_name=component_name,
                footprint=footprint,
            )
            if isinstance(self.input, EeSymbol)
            else logging.error("Unknown input symbol format")
        )

    def export(self, ato_full_path: str) -> str:
        # Get the directory of the file
        ato_dir = Path(ato_full_path).parent
        ato_dir.mkdir(parents=True, exist_ok=True)
        log.log(level=logging.INFO, msg=ato_full_path)
        with open(file=ato_full_path, mode="w", encoding="utf-8") as my_lib:
            my_lib.write(self.output)
            log.log(level=logging.INFO, msg="ATO file written")




# Global imports
import logging
import re
from dataclasses import dataclass, field
from enum import Enum
from typing import List, Union

from pydantic import BaseModel, validator


class SvgPathMoveTo(BaseModel):
    start_x: float
    start_y: float


class SvgPathLineTo(BaseModel):
    pos_x: float
    pos_y: float


class SvgPathEllipticalArc(BaseModel):
    radius_x: float
    radius_y: float
    x_axis_rotation: float
    flag_large_arc: bool
    flag_sweep: bool
    end_x: float
    end_y: float


class SvgPathClosePath(BaseModel):
    ...


svg_path_handlers = {
    "M": (SvgPathMoveTo, 2),
    "A": (SvgPathEllipticalArc, 7),
    "L": (SvgPathLineTo, 2),
    "Z": (SvgPathClosePath, 0),
}


def parse_svg_path(svg_path: str) -> list:
    if not svg_path.endswith(" "):
        svg_path += " "
    svg_path = svg_path.replace(",", " ")

    path_splited = re.findall(r"([a-zA-Z])([ ,\-\+.\d]+)", svg_path)

    parsed_path = []
    for path_command in path_splited:
        if cmd_class_info := svg_path_handlers.get(path_command[0]):
            cmd_class, cmd_nb_arguments = cmd_class_info
            arguments = path_command[1].strip().split(" ")
            # if multiple (x y) in a command
            parsed_path.extend(
                cmd_class(
                    **dict(
                        zip(cmd_class.__fields__, arguments[i : i + cmd_nb_arguments])
                    )
                )
                for i in range(0, len(arguments), cmd_nb_arguments or 1)
            )
        else:
            logging.warning("SVG command path not supported")

    return parsed_path


if __name__ == "__main__":
    path_1 = "M 400.067 299.929 A 4 3.9 0 1 1 408.032 299.934 L 5 6 5 7 Z"
    path_0 = "M 5 -6 L -5 0 L 5 6 Z"
    path_2 = "M -5 3 L -2 0 L -5 -3"

    print(parse_svg_path(svg_path=path_0))



from setuptools import find_packages, setup

with open("README.md") as fh:
    long_description = fh.read()

production_dependencies = ["pydantic>=2.0.0", "requests>2.0.0"]

development_dependencies = [
    "pre-commit>=2.17.0",
]

with open("requirements.txt", "w", encoding="utf-8") as f:
    f.write("\n".join(production_dependencies + development_dependencies))

setup(
    name="easyeda2ato",
    description=(
        "A Python script that convert any electronic components from LCSC or EasyEDA to"
        " a Kicad library"
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    version="0.2.3",
    author="uPesy",
    author_email="contact@upesy.com",
    url="https://github.com/uPesy/easyeda2kicad.py",
    project_urls={
        "Code": "https://github.com/uPesy/easyeda2kicad.py",
    },
    # download_url='',
    license="AGPL-3.0",
    py_modules=["easyeda2kicad"],
    platforms="any",
    packages=find_packages(),
    package_dir={"easyeda2kicad": "easyeda2kicad"},
    entry_points={"console_scripts": ["easyeda2kicad = easyeda2kicad.__main__:main"]},
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=production_dependencies,
    extras_require={"dev": development_dependencies},
    zip_safe=False,
    keywords="easyeda kicad library conversion",
    classifiers=[
        # More information at https://pypi.org/classifiers/.
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",
    ],
)



# Global imports
import argparse
import logging
import os
import re
import sys
from textwrap import dedent
from typing import List
from pathlib import Path

from easyeda2kicad import __version__
from easyeda2kicad.easyeda.easyeda_api import EasyedaApi
from easyeda2kicad.easyeda.easyeda_importer import (
    Easyeda3dModelImporter,
    EasyedaFootprintImporter,
    EasyedaSymbolImporter,
)
from easyeda2kicad.easyeda.parameters_easyeda import EeSymbol
from easyeda2kicad.helpers import (
    add_component_in_symbol_lib_file,
    get_local_config,
    id_already_in_symbol_lib,
    set_logger,
    update_component_in_symbol_lib_file,
)
from easyeda2kicad.kicad.export_kicad_3d_model import Exporter3dModelKicad
from easyeda2kicad.kicad.export_kicad_footprint import ExporterFootprintKicad
from easyeda2kicad.kicad.export_kicad_symbol import ExporterSymbolKicad
from easyeda2kicad.kicad.parameters_kicad_symbol import KicadVersion
from easyeda2kicad.atopile.export_ato import ExporterAto, sanitize_name


def get_parser() -> argparse.ArgumentParser:

    parser = argparse.ArgumentParser(
        description=(
            "A Python script that convert any electronic components from LCSC or"
            " EasyEDA to a Kicad library"
        )
    )

    parser.add_argument("--lcsc_id", help="LCSC id", required=True, type=str)

    parser.add_argument(
        "--ato", help="Get atopile file definition of this id", required=False, action="store_true"
    )

    parser.add_argument(
        "--ato_file_path",
        required=False,
        metavar="file.ato",
        help="Output dir for .ato file",
        type=str,
    )

    parser.add_argument(
        "--symbol", help="Get symbol of this id", required=False, action="store_true"
    )

    parser.add_argument(
        "--footprint",
        help="Get footprint of this id",
        required=False,
        action="store_true",
    )

    parser.add_argument(
        "--3d",
        help="Get the 3d model of this id",
        required=False,
        action="store_true",
    )

    parser.add_argument(
        "--full",
        help="Get the symbol, footprint and 3d model of this id",
        required=False,
        action="store_true",
    )

    parser.add_argument(
        "--output",
        required=False,
        metavar="file.kicad_sym",
        help="Output file",
        type=str,
    )

    parser.add_argument(
        "--overwrite",
        required=False,
        help=(
            "overwrite symbol and footprint lib if there is already a component with"
            " this lcsc_id"
        ),
        action="store_true",
    )

    parser.add_argument(
        "--v5",
        required=False,
        help="Convert library in legacy format for KiCad 5.x",
        action="store_true",
    )

    parser.add_argument(
        "--project-relative",
        required=False,
        help="Sets the 3D file path stored relative to the project",
        action="store_true",
    )

    parser.add_argument(
        "--debug",
        help="set the logging level to debug",
        required=False,
        default=False,
        action="store_true",
    )

    return parser


def valid_arguments(arguments: dict) -> bool:

    if not arguments["lcsc_id"].startswith("C"):
        logging.error("lcsc_id should start by C....")
        return False

    if arguments["full"]:
        arguments["ato"], arguments["symbol"], arguments["footprint"], arguments["3d"] = True, True, True, True

    if not any([arguments["ato"], arguments["symbol"], arguments["footprint"], arguments["3d"]]):
        logging.error(
            "Missing action arguments\n"
            "  easyeda2kicad --lcsc_id=C2040 --footprint\n"
            "  easyeda2kicad --lcsc_id=C2040 --symbol"
        )
        return False

    kicad_version = KicadVersion.v5 if arguments.get("v5") else KicadVersion.v6
    arguments["kicad_version"] = kicad_version

    if arguments["project_relative"] and not arguments["output"]:
        logging.error(
            "A project specific library path should be given with --output option when"
            " using --project-relative option\nFor example: easyeda2kicad"
            " --lcsc_id=C2040 --full"
            " --output=C:/Users/your_username/Documents/Kicad/6.0/projects/my_project"
            " --project-relative"
        )
        return False

    if arguments["output"]:
        base_folder = "/".join(arguments["output"].replace("\\", "/").split("/")[:-1])
        lib_name = (
            arguments["output"]
            .replace("\\", "/")
            .split("/")[-1]
            .split(".lib")[0]
            .split(".kicad_sym")[0]
        )

        if not os.path.isdir(base_folder):
            logging.error(f"Can't find the folder : {base_folder}")
            return False
    else:
        default_folder = os.path.join(
            os.path.expanduser("~"),
            "Documents",
            "Kicad",
            "easyeda2kicad",
        )
        if not os.path.isdir(default_folder):
            os.mkdir(default_folder)

        base_folder = default_folder
        lib_name = "easyeda2kicad"
        arguments["use_default_folder"] = True

    arguments["output"] = f"{base_folder}/{lib_name}"

    # Create new footprint folder if it does not exist
    if not os.path.isdir(f"{arguments['output']}.pretty"):
        os.mkdir(f"{arguments['output']}.pretty")
        logging.info(f"Create {lib_name}.pretty footprint folder in {base_folder}")

    # Create new 3d model folder if don't exist
    if not os.path.isdir(f"{arguments['output']}.3dshapes"):
        os.mkdir(f"{arguments['output']}.3dshapes")
        logging.info(f"Create {lib_name}.3dshapes 3D model folder in {base_folder}")

    lib_extension = "kicad_sym" if kicad_version == KicadVersion.v6 else "lib"
    if not os.path.isfile(f"{arguments['output']}.{lib_extension}"):
        with open(
            file=f"{arguments['output']}.{lib_extension}", mode="w+", encoding="utf-8"
        ) as my_lib:
            my_lib.write(
                dedent(
                    """\
                (kicad_symbol_lib
                  (version 20211014)
                  (generator https://github.com/uPesy/easyeda2kicad.py)
                )"""
                )
                if kicad_version == KicadVersion.v6
                else "EESchema-LIBRARY Version 2.4\n#encoding utf-8\n"
            )
        logging.info(f"Create {lib_name}.{lib_extension} symbol lib in {base_folder}")

    return True


def delete_component_in_symbol_lib(
    lib_path: str, component_id: str, component_name: str
) -> None:
    with open(file=lib_path, encoding="utf-8") as f:
        current_lib = f.read()
        new_data = re.sub(
            rf'(#\n# {component_name}\n#\n.*?F6 "{component_id}".*?ENDDEF\n)',
            "",
            current_lib,
            flags=re.DOTALL,
        )

    with open(file=lib_path, mode="w", encoding="utf-8") as my_lib:
        my_lib.write(new_data)


def fp_already_in_footprint_lib(lib_path: str, package_name: str) -> bool:
    if os.path.isfile(f"{lib_path}/{package_name}.kicad_mod"):
        logging.warning(f"The footprint for this id is already in {lib_path}")
        return True
    return False


def main(argv: List[str] = sys.argv[1:]) -> int:
    print(f"-- easyeda2kicad.py v{__version__} --")

    # cli interface
    parser = get_parser()
    try:
        args = parser.parse_args(argv)
    except SystemExit as err:
        return err.code
    arguments = vars(args)

    if arguments["debug"]:
        set_logger(log_file=None, log_level=logging.DEBUG)
    else:
        set_logger(log_file=None, log_level=logging.INFO)

    if not valid_arguments(arguments=arguments):
        return 1

    # conf = get_local_config()

    component_id = arguments["lcsc_id"]
    kicad_version = arguments["kicad_version"]
    sym_lib_ext = "kicad_sym" if kicad_version == KicadVersion.v6 else "lib"

    # Get CAD data of the component using easyeda API
    api = EasyedaApi()
    cad_data = api.get_cad_data_of_component(lcsc_id=component_id)

    # API returned no data
    if not cad_data:
        logging.error(f"Failed to fetch data from EasyEDA API for part {component_id}")
        return 1


    # ---------------- ATOPILE ----------------
    if arguments["ato"]:
        importer = EasyedaSymbolImporter(easyeda_cp_cad_data=cad_data)
        easyeda_symbol: EeSymbol = importer.get_symbol()

        component_name=sanitize_name(easyeda_symbol.info.name)
        # ato file path should be the the base directory of output argument /elec/src
        ato_full_path = f"{arguments['ato_file_path']}/{component_name}.ato"
        is_ato_already_in_lib_folder = os.path.isfile(ato_full_path)

        if not arguments["overwrite"] and is_ato_already_in_lib_folder:
            logging.error("Use --overwrite to update the older ato file")
            return 1

        footprint_importer = EasyedaFootprintImporter(easyeda_cp_cad_data=cad_data)
        easyeda_footprint = footprint_importer.get_footprint()
        package_name=easyeda_footprint.info.name

        exporter = ExporterAto(
            symbol = easyeda_symbol,
            component_id = component_id,
            component_name = component_name,
            footprint = package_name
        )

        exporter.export(
            ato_full_path = ato_full_path
        )


        logging.info(
            f"Created Atopile file for ID : {component_id}\n"
            f"       Symbol name : {easyeda_symbol.info.name}\n"
            f"       Library path : {ato_full_path}"
        )


    # ---------------- SYMBOL ----------------
    if arguments["symbol"]:
        importer = EasyedaSymbolImporter(easyeda_cp_cad_data=cad_data)
        easyeda_symbol: EeSymbol = importer.get_symbol()


        is_id_already_in_symbol_lib = id_already_in_symbol_lib(
            lib_path=f"{arguments['output']}.{sym_lib_ext}",
            component_name=easyeda_symbol.info.name,
            kicad_version=kicad_version,
        )

        if not arguments["overwrite"] and is_id_already_in_symbol_lib:
            logging.error("Use --overwrite to update the older symbol lib")
            return 1

        exporter = ExporterSymbolKicad(
            symbol=easyeda_symbol, kicad_version=kicad_version
        )

        kicad_symbol_lib = exporter.export(
            footprint_lib_name=arguments["output"].split("/")[-1].split(".")[0],
        )

        if is_id_already_in_symbol_lib:
            update_component_in_symbol_lib_file(
                lib_path=f"{arguments['output']}.{sym_lib_ext}",
                component_name=easyeda_symbol.info.name,
                component_content=kicad_symbol_lib,
                kicad_version=kicad_version,
            )
        else:
            add_component_in_symbol_lib_file(
                lib_path=f"{arguments['output']}.{sym_lib_ext}",
                component_content=kicad_symbol_lib,
                kicad_version=kicad_version,
            )

        logging.info(
            f"Created Kicad symbol for ID : {component_id}\n"
            f"       Symbol name : {easyeda_symbol.info.name}\n"
            f"       Library path : {arguments['output']}.{sym_lib_ext}"
        )

    # ---------------- FOOTPRINT ----------------
    if arguments["footprint"]:
        importer = EasyedaFootprintImporter(easyeda_cp_cad_data=cad_data)
        easyeda_footprint = importer.get_footprint()

        is_id_already_in_footprint_lib = fp_already_in_footprint_lib(
            lib_path=f"{arguments['output']}.pretty",
            package_name=easyeda_footprint.info.name,
        )
        if not arguments["overwrite"] and is_id_already_in_footprint_lib:
            logging.error("Use --overwrite to replace the older footprint lib")
            return 1

        ki_footprint = ExporterFootprintKicad(footprint=easyeda_footprint)
        footprint_filename = f"{easyeda_footprint.info.name}.kicad_mod"
        footprint_path = f"{arguments['output']}.pretty"
        model_3d_path = f"{arguments['output']}.3dshapes".replace("\\", "/").replace(
            "./", "/"
        )

        if arguments.get("use_default_folder"):
            model_3d_path = "${EASYEDA2KICAD}/easyeda2kicad.3dshapes"
        if arguments["project_relative"]:
            model_3d_path = "${KIPRJMOD}" + model_3d_path

        ki_footprint.export(
            footprint_full_path=f"{footprint_path}/{footprint_filename}",
            model_3d_path=model_3d_path,
        )

        logging.info(
            f"Created Kicad footprint for ID: {component_id}\n"
            f"       Footprint name: {easyeda_footprint.info.name}\n"
            f"       Footprint path: {os.path.join(footprint_path, footprint_filename)}"
        )

    # ---------------- 3D MODEL ----------------
    if arguments["3d"]:
        exporter = Exporter3dModelKicad(
            model_3d=Easyeda3dModelImporter(
                easyeda_cp_cad_data=cad_data, download_raw_3d_model=True
            ).output
        )
        exporter.export(lib_path=arguments["output"])
        if exporter.output or exporter.output_step:
            filename_wrl = f"{exporter.output.name}.wrl"
            filename_step = f"{exporter.output.name}.step"
            lib_path = f"{arguments['output']}.3dshapes"

            logging.info(
                f"Created 3D model for ID: {component_id}\n"
                f"       3D model name: {exporter.output.name}\n"
                + (
                    "       3D model path (wrl):"
                    f" {os.path.join(lib_path, filename_wrl)}\n"
                    if filename_wrl
                    else ""
                )
                + (
                    "       3D model path (step):"
                    f" {os.path.join(lib_path, filename_step)}\n"
                    if filename_step
                    else ""
                )
            )

        # logging.info(f"3D model: {os.path.join(lib_path, filename)}")

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



# Global imports
import logging
from typing import Callable, List, Tuple, Union

from easyeda2kicad.easyeda.parameters_easyeda import (
    EasyedaPinType,
    EeSymbol,
    EeSymbolArc,
    EeSymbolBbox,
    EeSymbolCircle,
    EeSymbolEllipse,
    EeSymbolPath,
    EeSymbolPin,
    EeSymbolPolygon,
    EeSymbolPolyline,
    EeSymbolRectangle,
)
from easyeda2kicad.easyeda.svg_path_parser import SvgPathEllipticalArc, SvgPathMoveTo
from easyeda2kicad.helpers import get_middle_arc_pos
from easyeda2kicad.kicad.export_kicad_footprint import compute_arc
from easyeda2kicad.kicad.parameters_kicad_symbol import *

ee_pin_type_to_ki_pin_type = {
    EasyedaPinType.unspecified: KiPinType.unspecified,
    EasyedaPinType._input: KiPinType._input,
    EasyedaPinType.output: KiPinType.output,
    EasyedaPinType.bidirectional: KiPinType.bidirectional,
    EasyedaPinType.power: KiPinType.power_in,
}


def px_to_mil(dim: Union[int, float]) -> int:
    return int(10 * dim)


def px_to_mm(dim: Union[int, float]) -> float:
    return 10.0 * dim * 0.0254


def convert_ee_pins(
    ee_pins: List[EeSymbolPin], ee_bbox: EeSymbolBbox, kicad_version: KicadVersion
) -> List[KiSymbolPin]:

    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm
    # pin_spacing = (
    #     KiExportConfigV5.PIN_SPACING.value
    #     if kicad_version == KicadVersion.v5
    #     else KiExportConfigV6.PIN_SPACING.value
    # )

    kicad_pins = []
    for ee_pin in ee_pins:
        pin_length = abs(int(float(ee_pin.pin_path.path.split("h")[-1])))

        ki_pin = KiSymbolPin(
            name=ee_pin.name.text.replace(" ", ""),
            number=ee_pin.settings.spice_pin_number.replace(" ", ""),
            style=KiPinStyle.line,
            length=to_ki(pin_length),
            type=ee_pin_type_to_ki_pin_type[ee_pin.settings.type],
            orientation=ee_pin.settings.rotation,
            pos_x=to_ki(int(ee_pin.settings.pos_x) - int(ee_bbox.x)),
            pos_y=-to_ki(int(ee_pin.settings.pos_y) - int(ee_bbox.y)),
        )

        if ee_pin.dot.is_displayed and ee_pin.clock.is_displayed:
            ki_pin.style = KiPinStyle.inverted_clock
        elif ee_pin.dot.is_displayed:
            ki_pin.style = KiPinStyle.inverted
        elif ee_pin.clock.is_displayed:
            ki_pin.style = KiPinStyle.clock

        # Deal with different pin length
        # if ee_pin.settings.rotation == 0:
        #     ki_pin.pos_x -= to_ki(pin_length) - pin_spacing
        # elif ee_pin.settings.rotation == 180:
        #     ki_pin.pos_x += to_ki(pin_length) - pin_spacing
        # elif ee_pin.settings.rotation == 90:
        #     ki_pin.pos_y -= to_ki(pin_length) - pin_spacing
        # elif ee_pin.settings.rotation == 270:
        #     ki_pin.pos_y += to_ki(pin_length) - pin_spacing

        kicad_pins.append(ki_pin)

    return kicad_pins


def convert_ee_rectangles(
    ee_rectangles: List[EeSymbolRectangle],
    ee_bbox: EeSymbolBbox,
    kicad_version: KicadVersion,
) -> List[KiSymbolRectangle]:

    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm

    kicad_rectangles = []
    for ee_rectangle in ee_rectangles:
        ki_rectangle = KiSymbolRectangle(
            pos_x0=to_ki(int(ee_rectangle.pos_x) - int(ee_bbox.x)),
            pos_y0=-to_ki(int(ee_rectangle.pos_y) - int(ee_bbox.y)),
        )
        ki_rectangle.pos_x1 = to_ki(int(ee_rectangle.width)) + ki_rectangle.pos_x0
        ki_rectangle.pos_y1 = -to_ki(int(ee_rectangle.height)) + ki_rectangle.pos_y0

        kicad_rectangles.append(ki_rectangle)

    return kicad_rectangles


def convert_ee_circles(
    ee_circles: List[EeSymbolCircle], ee_bbox: EeSymbolBbox, kicad_version: KicadVersion
):
    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm

    return [
        KiSymbolCircle(
            pos_x=to_ki(int(ee_circle.center_x) - int(ee_bbox.x)),
            pos_y=-to_ki(int(ee_circle.center_y) - int(ee_bbox.y)),
            radius=to_ki(ee_circle.radius),
            background_filling=ee_circle.fill_color,
        )
        for ee_circle in ee_circles
    ]


def convert_ee_ellipses(
    ee_ellipses: List[EeSymbolEllipse],
    ee_bbox: EeSymbolBbox,
    kicad_version: KicadVersion,
) -> List[KiSymbolCircle]:
    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm

    # Ellipses are not supported in Kicad -> If it's not a real ellipse, but just a circle
    return [
        KiSymbolCircle(
            pos_x=to_ki(int(ee_ellipses.center_x) - int(ee_bbox.x)),
            pos_y=-to_ki(int(ee_ellipses.center_y) - int(ee_bbox.y)),
            radius=to_ki(ee_ellipses.radius_x),
        )
        for ee_ellipses in ee_ellipses
        if ee_ellipses.radius_x == ee_ellipses.radius_y
    ]


def convert_ee_arcs(
    ee_arcs: List[EeSymbolArc], ee_bbox: EeSymbolBbox, kicad_version: KicadVersion
) -> List[KiSymbolArc]:
    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm

    kicad_arcs = []
    for ee_arc in ee_arcs:
        if not (
            isinstance(ee_arc.path[0], SvgPathMoveTo)
            or isinstance(ee_arc.path[1], SvgPathEllipticalArc)
        ):
            logging.error("Can't convert this arc")
        else:
            ki_arc = KiSymbolArc(
                radius=to_ki(
                    max(ee_arc.path[1].radius_x, ee_arc.path[1].radius_y)
                ),  # doesn't support elliptical arc
                angle_start=ee_arc.path[1].x_axis_rotation,
                start_x=to_ki(ee_arc.path[0].start_x - ee_bbox.x),
                start_y=to_ki(ee_arc.path[0].start_y - ee_bbox.y),
                end_x=to_ki(ee_arc.path[1].end_x - ee_bbox.x),
                end_y=to_ki(ee_arc.path[1].end_y - ee_bbox.y),
            )

            center_x, center_y, angle_end = compute_arc(
                start_x=ki_arc.start_x,
                start_y=ki_arc.start_y,
                radius_x=to_ki(ee_arc.path[1].radius_x),
                radius_y=to_ki(ee_arc.path[1].radius_y),
                angle=ki_arc.angle_start,
                large_arc_flag=ee_arc.path[1].flag_large_arc,
                sweep_flag=ee_arc.path[1].flag_sweep,
                end_x=ki_arc.end_x,
                end_y=ki_arc.end_y,
            )
            ki_arc.center_x = center_x
            ki_arc.center_y = center_y if ee_arc.path[1].flag_large_arc else -center_y
            ki_arc.angle_end = (
                (360 - angle_end) if ee_arc.path[1].flag_large_arc else angle_end
            )

            ki_arc.middle_x, ki_arc.middle_y = get_middle_arc_pos(
                center_x=ki_arc.center_x,
                center_y=ki_arc.center_y,
                radius=ki_arc.radius,
                angle_start=ki_arc.angle_start,
                angle_end=ki_arc.angle_end,
            )

            ki_arc.start_y = (
                ki_arc.start_y if ee_arc.path[1].flag_large_arc else -ki_arc.start_y
            )
            ki_arc.end_y = (
                ki_arc.end_y if ee_arc.path[1].flag_large_arc else -ki_arc.end_y
            )

            kicad_arcs.append(ki_arc)

    return kicad_arcs


def convert_ee_polylines(
    ee_polylines: List[Union[EeSymbolPolyline, EeSymbolPolygon]],
    ee_bbox: EeSymbolBbox,
    kicad_version: KicadVersion,
) -> List[KiSymbolPolygon]:

    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm
    kicad_polygons = []
    for ee_polyline in ee_polylines:
        raw_pts = ee_polyline.points.split(" ")
        # print(raw_pts)
        x_points = [
            to_ki(int(float(raw_pts[i])) - int(ee_bbox.x))
            for i in range(0, len(raw_pts), 2)
        ]
        y_points = [
            -to_ki(int(float(raw_pts[i])) - int(ee_bbox.y))
            for i in range(1, len(raw_pts), 2)
        ]

        if isinstance(ee_polyline, EeSymbolPolygon) or ee_polyline.fill_color:
            x_points.append(x_points[0])
            y_points.append(y_points[0])

        kicad_polygon = KiSymbolPolygon(
            points=[
                [x_points[i], y_points[i]]
                for i in range(min(len(x_points), len(y_points)))
            ],
            points_number=min(len(x_points), len(y_points)),
            is_closed=x_points[0] == x_points[-1] and y_points[0] == y_points[-1],
        )

        kicad_polygons.append(kicad_polygon)

    return kicad_polygons


def convert_ee_polygons(
    ee_polygons: List[EeSymbolPolygon],
    ee_bbox: EeSymbolBbox,
    kicad_version: KicadVersion,
) -> List[KiSymbolPolygon]:
    return convert_ee_polylines(
        ee_polylines=ee_polygons, ee_bbox=ee_bbox, kicad_version=kicad_version
    )


def convert_ee_paths(
    ee_paths: List[EeSymbolPath], ee_bbox: EeSymbolBbox, kicad_version: KicadVersion
) -> Tuple[List[KiSymbolPolygon], List[KiSymbolPolygon]]:
    kicad_polygons = []
    kicad_beziers = []
    to_ki: Callable = px_to_mil if kicad_version == KicadVersion.v5 else px_to_mm

    for ee_path in ee_paths:
        raw_pts = ee_path.paths.split(" ")

        x_points = []
        y_points = []

        # Small svg path parser : doc -> https://www.w3.org/TR/SVG11/paths.html#PathElement

        for i in range(len(raw_pts)):
            if raw_pts[i] in ["M", "L"]:
                x_points.append(to_ki(int(float(raw_pts[i + 1])) - int(ee_bbox.x)))
                y_points.append(-to_ki(int(float(raw_pts[i + 2])) - int(ee_bbox.y)))
                i += 2
            elif raw_pts[i] == "Z":
                x_points.append(x_points[0])
                y_points.append(y_points[0])
            elif raw_pts[i] == "C":
                ...
                # TODO : Add bezier support

        # if ee_path.fill_color:
        #     x_points.append(x_points[0])
        #     y_points.append(y_points[0])

        ki_polygon = KiSymbolPolygon(
            points=[
                [x_points[i], y_points[i]]
                for i in range(min(len(x_points), len(y_points)))
            ],
            points_number=min(len(x_points), len(y_points)),
            is_closed=x_points[0] == x_points[-1] and y_points[0] == y_points[-1],
        )

        kicad_polygons.append(ki_polygon)

    return kicad_polygons, kicad_beziers


def convert_to_kicad(ee_symbol: EeSymbol, kicad_version: KicadVersion) -> KiSymbol:

    ki_info = KiSymbolInfo(
        name=ee_symbol.info.name,
        prefix=ee_symbol.info.prefix.replace("?", ""),
        package=ee_symbol.info.package,
        manufacturer=ee_symbol.info.manufacturer,
        datasheet=ee_symbol.info.datasheet,
        lcsc_id=ee_symbol.info.lcsc_id,
        jlc_id=ee_symbol.info.jlc_id,
    )

    kicad_symbol = KiSymbol(
        info=ki_info,
        pins=convert_ee_pins(
            ee_pins=ee_symbol.pins, ee_bbox=ee_symbol.bbox, kicad_version=kicad_version
        ),
        rectangles=convert_ee_rectangles(
            ee_rectangles=ee_symbol.rectangles,
            ee_bbox=ee_symbol.bbox,
            kicad_version=kicad_version,
        ),
        circles=convert_ee_circles(
            ee_circles=ee_symbol.circles,
            ee_bbox=ee_symbol.bbox,
            kicad_version=kicad_version,
        ),
        arcs=convert_ee_arcs(
            ee_arcs=ee_symbol.arcs, ee_bbox=ee_symbol.bbox, kicad_version=kicad_version
        ),
    )
    kicad_symbol.circles += convert_ee_ellipses(
        ee_ellipses=ee_symbol.ellipses,
        ee_bbox=ee_symbol.bbox,
        kicad_version=kicad_version,
    )

    kicad_symbol.polygons, kicad_symbol.beziers = convert_ee_paths(
        ee_paths=ee_symbol.paths, ee_bbox=ee_symbol.bbox, kicad_version=kicad_version
    )
    kicad_symbol.polygons += convert_ee_polylines(
        ee_polylines=ee_symbol.polylines,
        ee_bbox=ee_symbol.bbox,
        kicad_version=kicad_version,
    )
    kicad_symbol.polygons += convert_ee_polygons(
        ee_polygons=ee_symbol.polygons,
        ee_bbox=ee_symbol.bbox,
        kicad_version=kicad_version,
    )

    return kicad_symbol


def tune_footprint_ref_path(ki_symbol: KiSymbol, footprint_lib_name: str):
    ki_symbol.info.package = f"{footprint_lib_name}:{ki_symbol.info.package}"


class ExporterSymbolKicad:
    def __init__(self, symbol, kicad_version: KicadVersion):
        self.input: EeSymbol = symbol
        self.version = kicad_version
        self.output = (
            convert_to_kicad(ee_symbol=self.input, kicad_version=kicad_version)
            if isinstance(self.input, EeSymbol)
            else logging.error("Unknown input symbol format")
        )

    def export(self, footprint_lib_name: str) -> str:

        tune_footprint_ref_path(
            ki_symbol=self.output,
            footprint_lib_name=footprint_lib_name,
        )
        return self.output.export(kicad_version=self.version)
